package pwc.taxtech.atms.controller;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;
import pwc.taxtech.atms.dto.ReturnData;
import pwc.taxtech.atms.entity.OperationLogFileType;
import pwc.taxtech.atms.service.impl.OperationLogFileTypeServiceImpl;
import pwc.taxtech.atms.thirdparty.ExcelUtil;

import javax.servlet.http.HttpServletResponse;
import java.io.OutputStream;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Controller
@RequestMapping("/api/v1/operLogFileTypes")
public class OperationLogFileTypeController {

    @Autowired
    private OperationLogFileTypeServiceImpl operationLogFileTypeService;

    /**
     * 查询文件档案操作日志列表
     * @return
     */
    @RequestMapping("selectList")
    @ResponseBody
    public ReturnData selectFileTypesList(){
        List<OperationLogFileType> operationLogFileType = operationLogFileTypeService.selectFileTypesList();
        ReturnData returnData = new ReturnData();
        returnData.setItems(operationLogFileType);
        returnData.setTotalCount(operationLogFileType.size());
        return returnData;
    }

    /**
     * 根据 id 数组来查询相关日志
     * @return
     */
    @RequestMapping("/selectListForLog")
    @ResponseBody
    public ReturnData selectListForLog(@RequestBody OperationLogFileType operationLogFileType){
        List<OperationLogFileType> operationLogFileTypes = operationLogFileTypeService.selectListForLog(operationLogFileType.getFileTypeIds());
        ReturnData returnData = new ReturnData();
        returnData.setItems(operationLogFileTypes);
        returnData.setTotalCount(operationLogFileTypes.size());
        return returnData;
    }

    /**
     * 添加文件档案操作日志
     * @param operationLogFileType
     * @return
     */
    @RequestMapping("add")
    @ResponseBody
    public boolean addFileTypes(OperationLogFileType operationLogFileType){
        return operationLogFileTypeService.addFileTypesList(operationLogFileType);
    }

    /**
     * 删除文件档案操作日志
     * @param id
     * @return
     */
    @RequestMapping("delete")
    @ResponseBody
    public boolean deleteFileTypes(String id){
        return operationLogFileTypeService.deleteFileTypes(id);
    }

    /**
     * 更新文件档案操作日志
     * @param operationLogFileType
     * @return
     */
    @RequestMapping("edit")
    @ResponseBody
    public boolean editFileTypes(OperationLogFileType operationLogFileType){
        return operationLogFileTypeService.editFilesType(operationLogFileType);
    }

    /**
     * 导出文件档案操作日志
     * @param response
     */
    @RequestMapping("exportExcel")
    @ResponseBody
    public void exportExcelFile (HttpServletResponse response){
//    public ReturnData selectListForLog(@RequestBody OperationLogFileType operationLogFileType){
        try {
            Map<String, String> headers = new HashMap<String, String>();
            headers.put("id", "id");
//            headers.put("operation_content", "操作内容");
//            headers.put("module_name", "模块名称");
//            headers.put("operation_object", "操作对象");
            headers.put("operation_action", "操作action");
//            headers.put("original_state", "原始状态");
//            headers.put("update_state", "更新状态");
            headers.put("update_state", "操作内容");
            headers.put("operation_user", "操作者");
            headers.put("ip", "操作ip");
            headers.put("comment", "内容");
            headers.put("create_time", "创建时间");
            List<OperationLogFileType> fileTypes = operationLogFileTypeService.selectFileTypesList();
//            List<OperationLogFileType> operationLogFileTypes = operationLogFileTypeService.selectListForLog(operationLogFileType.getFileTypeIds());
            response.setContentType("multipart/form-data");
            response.setHeader("Content-Disposition", "attachment;fileName=" + new String("档案类型日志".getBytes("GB2312"), "ISO-8859-1"));
            OutputStream ouputStream = response.getOutputStream();
            ExcelUtil.exportExcel(headers, fileTypes, ouputStream);
        }catch(Exception e){
            e.printStackTrace();
        }
    }
}
