package pwc.taxtech.atms.service.impl;

import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.apache.commons.io.FileUtils;
import org.apache.poi.openxml4j.exceptions.InvalidFormatException;
import org.apache.poi.ss.usermodel.*;
import org.nutz.lang.Files;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.RequestParam;
import pwc.taxtech.atms.common.CommonUtils;
import pwc.taxtech.atms.dao.AssetDetailGroupMapper;
import pwc.taxtech.atms.dao.AssetGroupMapper;
import pwc.taxtech.atms.dao.CitAssetGroupResultMapper;
import pwc.taxtech.atms.dao.CitAssetsListMapper;
import pwc.taxtech.atms.dto.*;
import pwc.taxtech.atms.entity.*;
import pwc.taxtech.atms.vat.entity.AssetsList;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author zhikai.z.wei
 * 资产分类service
 */
@Service
public class AssetListServiceImpl extends BaseService {
    private static Logger logger = LoggerFactory.getLogger(AssetListServiceImpl.class);

    @Autowired
    private CitAssetsListMapper assetListMapper;

    @Autowired
    private CitAssetGroupResultMapper assetGroupResultMapper;

    @Autowired
    private AssetDetailGroupMapper assetDetailGroupMapper;

    @Autowired
    private AssetGroupMapper assetGroupMapper;

    @Autowired
    private FileService fileService;

    /**
     * 获取资产清单
     */
    public List<CitAssetsList> getAssetListData() throws Exception {
        logger.debug("获取资产一级分类相关数据");
        CitAssetsListExample assetGroupExample = new CitAssetsListExample();
        return assetListMapper.selectByExample(assetGroupExample);
    }

    /**
     * 根据资产类别（固定资产、长期待摊、无形资产）查询出结果
     */
    public List<CitAssetsListDto> getAssetResultList(Integer assetType, String projectId,Integer taxAccountCompare) throws Exception {
        logger.debug("根据资产类别（固定资产、长期待摊、无形资产）查询出结果");
        CitAssetsListExample assetListExample = new CitAssetsListExample();
        CitAssetsListExample.Criteria criteria = assetListExample.createCriteria();
        criteria.andProjectIdEqualTo(projectId);
        criteria.andAssetTypeEqualTo(assetType);
        criteria.andStatusEqualTo(1);
        if(taxAccountCompare != 2){
            criteria.andTaxAccountCompareEqualTo(taxAccountCompare);
        }
        List<CitAssetsList> citAssetsLists = assetListMapper.selectByExample(assetListExample);
        List<CitAssetsListDto> citAssetsListDtos = new ArrayList<>();
        for (CitAssetsList citAssetsList:citAssetsLists){
            CitAssetsListDto temp = new CitAssetsListDto();
            BeanUtils.copyProperties(citAssetsList,temp);
            citAssetsListDtos.add(temp);
        }
        return citAssetsListDtos;
    }

    /**
     * 根据资产类别（固定资产、长期待摊、无形资产）查询出结果
     */
    public PageInfo<CitAssetsListDto> getAssetResultListPage(CitAssetsListDto citAssetsListDto) throws Exception {
        logger.debug("根据资产类别（固定资产、长期待摊、无形资产）查询出结果");
        CitAssetsListExample assetListExample = new CitAssetsListExample();
        CitAssetsListExample.Criteria criteria = assetListExample.createCriteria();
        criteria.andProjectIdEqualTo(citAssetsListDto.getProjectId());
        criteria.andAssetTypeEqualTo(citAssetsListDto.getAssetType());
        criteria.andStatusEqualTo(1);
        if(citAssetsListDto.getTaxAccountCompare() != 2){
            criteria.andTaxAccountCompareEqualTo(citAssetsListDto.getTaxAccountCompare());
        }
        Page page = PageHelper.startPage(citAssetsListDto.getPageInfo().getPageIndex(),citAssetsListDto.getPageInfo().getPageSize());
        List<CitAssetsList> citAssetsLists = assetListMapper.selectByExample(assetListExample);
        List<CitAssetsListDto> citAssetsListDtos = new ArrayList<>();
        for (CitAssetsList citAssetsList:citAssetsLists){
            CitAssetsListDto temp = new CitAssetsListDto();
            BeanUtils.copyProperties(citAssetsList,temp);
            citAssetsListDtos.add(temp);
        }

        PageInfo<CitAssetsListDto> pageInfo =new PageInfo<>(citAssetsListDtos);
        pageInfo.setTotal(page.getTotal());
        pageInfo.setPageNum(citAssetsListDto.getPageInfo().getPageIndex());
        return pageInfo;
    }

    public List<CitAssetGroupResultDto> getAssetGroupResultData(String projectId) throws Exception {
        logger.debug("根据projectId获取该资产类别相关数据");
        CitAssetGroupResultExample assetGroupResultExample = new CitAssetGroupResultExample();
        CitAssetGroupResultExample.Criteria criteria = assetGroupResultExample.createCriteria();
        criteria.andProjectIdEqualTo(projectId);
        List<CitAssetGroupResult> citAssetGroupResults = assetGroupResultMapper.selectByExample(assetGroupResultExample);
        List<CitAssetGroupResultDto> citAssetGroupResultDtos = new ArrayList<>();

        for (CitAssetGroupResult citAssetGroupResult:citAssetGroupResults){
            CitAssetGroupResultDto temp = new CitAssetGroupResultDto();
            BeanUtils.copyProperties(citAssetGroupResult,temp);
            citAssetGroupResultDtos.add(temp);
        }
        return citAssetGroupResultDtos;
    }

    /**
     * 获取资产一级分类
     *
     * @return
     */
    public List<AssetGroupDto> getAssetGroupListData() throws Exception {
        logger.debug("获取资产一级分类相关数据");
        AssetGroupExample assetGroupExample = new AssetGroupExample();
        AssetGroupExample.Criteria criteria = assetGroupExample.createCriteria();
        List<AssetGroup> assetGroups = assetGroupMapper.selectByExample(assetGroupExample);
        List<AssetGroupDto> assetGroupDtos = new ArrayList<>();
        for (AssetGroup assetGroup:assetGroups){
            AssetGroupDto temp = new AssetGroupDto();
            BeanUtils.copyProperties(assetGroup,temp);
            assetGroupDtos.add(temp);
        }
        return assetGroupDtos;
    }

    /**
     * 获取资产二级分类
     *
     * @return
     */
    public List<AssetDetailGroupDto> getAssetDetailGroupListData() throws Exception {
        logger.debug("获取资产一级分类相关数据");
        AssetDetailGroupExample assetDetailGroupExample = new AssetDetailGroupExample();
        AssetDetailGroupExample.Criteria criteria = assetDetailGroupExample.createCriteria();
        List<AssetDetailGroup> assetDetailGroups = assetDetailGroupMapper.selectByExample(assetDetailGroupExample);
        List<AssetDetailGroupDto> assetDetailGroupDtos = new ArrayList<>();
        for (AssetDetailGroup assetDetailGroup:assetDetailGroups){
            AssetDetailGroupDto temp = new AssetDetailGroupDto();
            BeanUtils.copyProperties(assetDetailGroup,temp);
            assetDetailGroupDtos.add(temp);
        }
        return assetDetailGroupDtos;
    }

    /**
     * 导入Excel业务逻辑
     *
     * @param inputStream
     * @param fileName
     * @return
     */
    public OperationResultDto assetsImport(InputStream inputStream, String fileName, String projectId) throws IOException, InvalidFormatException, ParseException {
        logger.debug("导入excel文件开始");
        //定义返回变量
        OperationResultDto<Object> importResult = new OperationResultDto<>();
        importResult.setResult(true);
        StringBuilder errorMsgSb = new StringBuilder("第");

        //通过输入流获取当前workbook
        Workbook workbook = fileService.getWorkbook(inputStream, fileName, "citAsset");
        if(workbook == null){
            importResult.setResult(false);
            importResult.setResultMsg("读取Excel出现内部错误");
            return importResult;
        }
        //目前资产导入只有一个Sheet页，所以只取第一个Sheet即可
        Sheet sheet = workbook.getSheetAt(0);
        if (sheet == null) {
            importResult.setResult(false);
            importResult.setResultMsg("读取Excel出现内部错误");
            return importResult;
        }

        //在循环取数据外定义资产类别map，防止map重新定义，不能过滤重复资产类别
        Set assetNameSet = new HashSet();
        List<CitAssetGroupResult> citAssetGroupResults = selectGroupResult(projectId);

        Integer period = CitCommonUtil.getPeriod();
        List<CitAssetsList> citAssetsLists = new ArrayList<>();

        //通过循环来完成sheet获取每一行每一列数据并进行相关逻辑处理
        for (int rowNum = sheet.getFirstRowNum() + 1; rowNum <= sheet.getLastRowNum(); rowNum++) {
            CitAssetsList citAsset = new CitAssetsList();
            citAsset.setId(idService.nextId());
            //赋值projectId
            citAsset.setProjectId(projectId);

            citAsset.setPeriod(Integer.valueOf(period));

            //获取该行数据
            Row rowData = sheet.getRow(rowNum);
            //因模板固定，则按照写死的做法处理数据单元格数据
            citAsset = cellToCitAsset(rowData, citAsset, assetNameSet, citAssetGroupResults);
            if(citAsset == null){
                errorMsgSb.append(rowNum+"，");
                continue;
            }
            if(!importResult.getResult()){
                importResult.setResultMsg("行数据为空或格式错误！");
            }
            //目前没有进行税法分类，标注状态
//            citAsset.setStatus(0);
            citAssetsLists.add(citAsset);

        }
        //在循环存储新的资产之前先删除原有资产
        deleteByExample(projectId);
        //循环存储新的资产
        int insertBatch = assetListMapper.insertBatch(citAssetsLists);
//        for (CitAssetsList citAsset : citAssetsLists){
//            assetListMapper.insertSelective(citAsset);
//        }
        //处理资产类别
        //1、根据projectId查询数据库中已有的资产类别，并和新导入的资产类别进行比较
//        for (CitAssetGroupResult groupResult : citAssetGroupResults) {
//            assetNameSet.remove(groupResult.getAssetName());
//        }
        Iterator<String> it = assetNameSet.iterator();
        while (it.hasNext()) {
            CitAssetGroupResult citAssetGroupResult = new CitAssetGroupResult();
            citAssetGroupResult.setId(idService.nextId());
            citAssetGroupResult.setProjectId(projectId);
            citAssetGroupResult.setPeriod(Integer.valueOf(period));
            citAssetGroupResult.setAssetName(it.next());
            assetGroupResultMapper.insertSelective(citAssetGroupResult);
        }
        importResult.setResult(true);
        return importResult;
    }

    /**
     * 把每一行的数据转换成实体，在此实体即为 CitAssetsList
     *
     * @param rowData
     * @return
     */
    private CitAssetsList cellToCitAsset(Row rowData, CitAssetsList citAsset, Set assetNameSet, List<CitAssetGroupResult> citAssetGroupResults ) throws ParseException {
        //获取单元格数据,并set进入实体

        //获取资产编号
        Object value = CitCommonUtil.getValue(rowData.getCell(1));
        if (value == null) {
            return null;
        }
        citAsset.setAssetNumber(value.toString());
        citAsset.setSerialNumber(String.valueOf(CitCommonUtil.getValue(rowData.getCell(2))));

        //获取资产描述
        citAsset.setAssetDescription(String.valueOf(CitCommonUtil.getValue(rowData.getCell(5))));
        //获取资产类别
        value = CitCommonUtil.getValue(rowData.getCell(7));
        if (value == null) {
            return null;
        }
        String assetName = value.toString();
        citAsset.setAssetGroupName(assetName);

        //获取购入日期
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        value = CitCommonUtil.getValue(rowData.getCell(15));
        citAsset.setBuyDate(value==null?null:sdf.parse(value.toString()));
        //获取开始折旧日期,DD没有提供，我们要根据购入日期和税法分类做预处理自己转化
//        citAsset.setDepreciationDate(sdf.parse(CitCommonUtil.getValue(rowData.getCell(15)).toString()));

        //获取折旧期限,接下来计算需要用到，所以赋给一个对象
        value = CitCommonUtil.getValue(rowData.getCell(30));
        Integer depreciationPeriod = Integer.valueOf(value==null?"0":value.toString());
        citAsset.setDepreciationPeriod(depreciationPeriod);

        //获取原值,接下来计算需要用到，所以赋给一个对象
        value = CitCommonUtil.getValue(rowData.getCell(16));
        if (value == null) {
            return null;
        }
        BigDecimal acquisitionValue = new BigDecimal(value.toString());
        citAsset.setAcquisitionValue(acquisitionValue);

        //获取原值调整值0----列数待定,滴滴无调整值
//        citAsset.setAdjustmentValue(new BigDecimal(CitCommonUtil.getValue(rowData.getCell(32)).toString()));
        //获取报废日期
        value = CitCommonUtil.getValue(rowData.getCell(37));
        citAsset.setDisposedDate(value==null?null:sdf.parse(value.toString()));
        citAsset.setScrapType(String.valueOf(CitCommonUtil.getValue(rowData.getCell(38))));
        //获取残值额，原为残值率现为残值额，接下来计算需要用到，所以赋给一个对象
        BigDecimal residualRate = new BigDecimal(CitCommonUtil.getValue(rowData.getCell(20)).toString());
        citAsset.setResidualRate(residualRate);

        //获取本年折旧额
        citAsset.setYearDepreciationAmount(new BigDecimal(CitCommonUtil.getValue(rowData.getCell(24)).toString()));
        // 获取本年调整额----列数待定 滴滴无本年调整额
//        citAsset.setYearAdjustmentAmount(new BigDecimal(CitCommonUtil.getValue(rowData.getCell(25)).toString()));

        //获取财务原值---计算方法：原值+原值调整值，暂时不考虑调整值
        citAsset.setAccountAcquisitionValue(acquisitionValue);
        //获取财务每月折旧额----（原值-残值额）/折旧期限
        citAsset.setAccountMonthDepreciationAmount(acquisitionValue.subtract(residualRate).divide(new BigDecimal(depreciationPeriod), 2));
        //获取财务本年折旧额
        citAsset.setAccountYearDepreciationAmount(new BigDecimal(CitCommonUtil.getValue(rowData.getCell(24)).toString()));
        //获取财务累计折旧额,接下来计算需要用到，所以赋给一个对象
        BigDecimal accountTotalDepreciationAmount = new BigDecimal(CitCommonUtil.getValue(rowData.getCell(25)).toString());
        citAsset.setAccountTotalDepreciationAmount(accountTotalDepreciationAmount);
        //年终剩余价值,原值-累计折旧
        citAsset.setYearEndValue(acquisitionValue.subtract(accountTotalDepreciationAmount));
        //过滤查询出的资产类别，若已对该资产类别进行过分类直接进行税务数据的计算
        List<CitAssetGroupResult> groupResults = citAssetGroupResults.stream()
                .filter(groupResult -> groupResult.getAssetName().equals(citAsset.getAssetGroupName())).collect(Collectors.toList());
        //通过判断过滤后的集合来确认是否已经对该资产类别进行过分类
        if(groupResults != null && groupResults.size()>0){
            //目前已经进行税法分类，标注状态
            citAsset.setStatus(1);
            AssetDetailGroup assetDetailGroup = selectAssetDetailGroup(groupResults.get(0).getAssetDetailGroupId());
//            Integer assetGroupType = selectAssetDetailGroup(groupResults.get(0).getAssetDetailGroupId()).getAssetGroupType();
            //进行税务的计算
            setCitAssetTaxData(citAsset,groupResults.get(0),assetDetailGroup);
        }else{
            //目前没有进行税法分类，标注状态
            citAsset.setStatus(0);
            //没有进行过分类，放入Set集合里面
            assetNameSet.add(assetName);
        }
        return citAsset;
    }


    /**
     * 根据projectId获取资产类别数据
     * @param projectId
     * @return
     */
    private List<CitAssetGroupResult> selectGroupResult(String projectId) {
        CitAssetGroupResultExample groupResultExample = new CitAssetGroupResultExample();
        CitAssetGroupResultExample.Criteria criteria = groupResultExample.createCriteria();
        criteria.andProjectIdEqualTo(projectId);
        return assetGroupResultMapper.selectByExample(groupResultExample);
    }

    /**
     * 根据projectId删除资产清单数据，覆盖导入操作时需使用该方法
     * @param projectId
     */
    public void deleteByExample(String projectId) {
        CitAssetsListExample citAssetsListExample = new CitAssetsListExample();
        CitAssetsListExample.Criteria criteria = citAssetsListExample.createCriteria();
        criteria.andProjectIdEqualTo(projectId);
        assetListMapper.deleteByExample(citAssetsListExample);
    }

    /**
     * 当分类选择完毕后，进行的业务逻辑操作
     * 1、保存资产类别的分类信息
     * 2、根据资产类别的分类计算税务的各种数据
     * @param assetGroupResults
     * @param saveGroupType
     * @param projectId
     * @return
     */
    public ApiResultDto saveAssetGroupInfo(List<CitAssetGroupResult> assetGroupResults, Integer saveGroupType, String projectId) {
        ApiResultDto resultDto = new ApiResultDto();
        for (CitAssetGroupResult assetGroupResult : assetGroupResults) {
            //根据主键修改资产类别有关数据，主要修改的字段为资产一级分类主键、资产二级分类主键
            assetGroupResultMapper.updateByPrimaryKeySelective(assetGroupResult);
            AssetDetailGroup assetDetailGroup = selectAssetDetailGroup(assetGroupResult.getAssetDetailGroupId());
//            Integer assetGroupType = selectAssetGroup(assetGroupResult.getAssetGroupId()).getAssetGroupType();
            List<CitAssetsList> citAssetsLists = selectAssetsList(projectId, assetGroupResult.getAssetName(),saveGroupType);
            citAssetsLists.stream().forEach(citAsset -> {
                setCitAssetTaxData(citAsset,assetGroupResult,assetDetailGroup);
            });
        }

//        List<CitAssetsList> citAssetsLists = selectAssetsList(projectId);
//        citAssetsLists.stream().forEach(asset -> {
//            for (int i = 0; i < assetGroupResults.size(); i++) {
//
//            }
//        });
        resultDto.setData(true);
        resultDto.setCode(1);
        return resultDto;
    }


    /**
     * 设置税务数据
     * @param citAsset
     * @param assetGroupResult
     * @param assetDetailGroup
     */
    public void setCitAssetTaxData(CitAssetsList citAsset,CitAssetGroupResult assetGroupResult,AssetDetailGroup assetDetailGroup){
        Calendar cal = Calendar.getInstance();
        //设置税法分类
        citAsset.setTaxGroupName(assetDetailGroup.getDetailGroupName());
        citAsset.setAssetDetailGroupId(assetDetailGroup.getId());
        Integer assetGroupType = assetDetailGroup.getAssetGroupType();
        //获取税务折旧期限,此值根据分类查数据库带出的值不在此进行操作
        citAsset.setTaxDepreciationPeriod(assetGroupResult.getTaxDepreciationPeriod());
        //获取截止去年累计折旧期间,此值根据分类查数据库带出的值不在此进行操作
        Integer tempPeriod = 0;
        cal.setTime(citAsset.getBuyDate());
        if(assetGroupType == 1 || assetGroupType == 2){
            tempPeriod = cal.get(Calendar.MONTH)+1;
        }else{
            tempPeriod = cal.get(Calendar.MONTH)+2;
        }
        citAsset.setTaxToLastYearDepreciationPeriod(tempPeriod);
        //获取截止本年累计折旧期间,此值根据分类查数据库带出的值不在此进行操作
        if(assetGroupType == 1 || assetGroupType == 2){
            tempPeriod = 12-cal.get(Calendar.MONTH)+1;
        }else{
            tempPeriod = 12-cal.get(Calendar.MONTH)+2;
        }
        citAsset.setTaxToCurrentYearDepreciationPeriod(tempPeriod);
        //获取本年折旧期间,此值根据分类查数据库带出的值
        citAsset.setTaxYearDepreciationPeriod(citAsset.getTaxToLastYearDepreciationPeriod()-citAsset.getTaxToCurrentYearDepreciationPeriod());
        //获取税务每月折旧额
        citAsset.setTaxMonthDepreciationAmount(citAsset.getAcquisitionValue().subtract(citAsset.getResidualRate()).divide(new BigDecimal(assetGroupResult.getTaxDepreciationPeriod()), 2));
        //获取税务截止本年累计折旧额
        citAsset.setTaxToCurrentYearDepreciationAmount(citAsset.getTaxMonthDepreciationAmount().multiply(new BigDecimal(citAsset.getTaxToCurrentYearDepreciationPeriod())));
        //获取税务本年折旧额
        citAsset.setTaxCurrentYearDepreciationAmount(citAsset.getTaxMonthDepreciationAmount().multiply(new BigDecimal(citAsset.getTaxYearDepreciationPeriod())));
        //获取累计差异
        citAsset.setTotalDifferenceAmount(citAsset.getTaxToCurrentYearDepreciationAmount().subtract(citAsset.getAccountTotalDepreciationAmount()));
        //获取当年差异
        citAsset.setYearDifferenceAmount(citAsset.getTaxCurrentYearDepreciationAmount().subtract(citAsset.getAccountYearDepreciationAmount()));
        //获取保留差异
        citAsset.setIsRetain(1);
        //获取资产一级分类的类型
        citAsset.setAssetType(assetGroupType);
        //设置税会比较值
        citAsset.setTaxAccountCompare(citAsset.getTaxCurrentYearDepreciationAmount().compareTo(citAsset.getAccountYearDepreciationAmount())==1?1:0);
        //更改资产状态，已进行分类并有效
        citAsset.setStatus(1);
        //根据主键修改资产清单数据
        assetListMapper.updateByPrimaryKeySelective(citAsset);
    }

    public ApiResultDto updateAssetResultList(List<CitAssetsList> assetsLists, String projectId){
        for(CitAssetsList citAsset:assetsLists){
            assetListMapper.updateByPrimaryKeySelective(citAsset);
        }
        return null;
    }

    /**
     * 获取符合条件(projectId、未进行分类)的资产清单数据
     * @param projectId
     * @return
     */
    public List<CitAssetsList> selectAssetsList(String projectId) {
        CitAssetsListExample assetListExample = new CitAssetsListExample();
        CitAssetsListExample.Criteria criteria = assetListExample.createCriteria();
        criteria.andProjectIdEqualTo(projectId);
        criteria.andStatusEqualTo(0);
        return assetListMapper.selectByExample(assetListExample);
    }

    /**
     * 获取符合条件(projectId、资产类别、未进行分类)的资产清单数据
     * @param projectId
     * @param assetName
     * @return
     */
    private List<CitAssetsList> selectAssetsList(String projectId, String assetName, Integer saveGroupType) {
        CitAssetsListExample assetListExample = new CitAssetsListExample();
        CitAssetsListExample.Criteria criteria = assetListExample.createCriteria();
        criteria.andProjectIdEqualTo(projectId);
        //0表示未进行分类的数据
        //当 saveGroupType为1时证明是初次导入进来时的数据需要进行状态的判断，当为其他或者为2时需要查询全部状态的数据
        if(saveGroupType == 1){

            criteria.andStatusEqualTo(0);
        }
        criteria.andAssetGroupNameEqualTo(assetName);
        return assetListMapper.selectByExample(assetListExample);
    }

    public void batchUpdateIsRetain(Integer assetType, String projectId, Integer isRetain){
        CitAssetsListExample citAssetsListExample = new CitAssetsListExample();
        CitAssetsListExample.Criteria criteria = citAssetsListExample.createCriteria();
        criteria.andProjectIdEqualTo(projectId);
        criteria.andAssetTypeEqualTo(assetType);
        CitAssetsList citAsset = new CitAssetsList();
        citAsset.setIsRetain(isRetain);
        assetListMapper.updateByExampleSelective(citAsset,citAssetsListExample);
    }

    /**
     * 根据主键获取资产一级分类
     * @param id
     * @return
     */
    private AssetGroup selectAssetGroup(Long id) {
        AssetGroupExample assetGroupExample = new AssetGroupExample();
        AssetGroupExample.Criteria criteria = assetGroupExample.createCriteria();
        return assetGroupMapper.selectByPrimaryKey(id);
    }

    private AssetDetailGroup selectAssetDetailGroup(Long id) {
        return assetDetailGroupMapper.selectByPrimaryKey(id);
    }
}
